#!/usr/bin/perl -w
#
# Author: Petter Reinholdtsen <pere@hungry.com>
# Date:   2001-08-23
#
# Extract packages for use with skolelinux CD from textfiles and dump
# the list on stdout.

use strict;

use Getopt::Std;
use File::Path;

use vars qw(%opts %available %excluded %included @wanted @missing $debug);

$debug = 0;

getopts("aem", \%opts);

load_available_packages();

load_tasks();

if ($opts{'e'}) {
    print_excluded_packages();
} elsif ($opts{'a'}) {
    print_all_packages();
} else {
    print_available_packages();
}

print_missing_packages() if ($opts{'m'});

sub apt {
    my $op = shift;

    my $aptsources = "./sources.list.skolelinux";
    my $aptdir  = "./tmp/apt";
    my @aptopts = ("Dir::Etc::sourcelist=$aptsources",
		   "Dir::State=$aptdir/state",
		   "Debug::NoLocking=true",
		   "Dir::Cache=$aptdir/cache");

    # Stupid apt-get and apt-cache do not understand the same arguments!
    # I have to map them to different formats to get both working.

    if ("update" eq $op) {
	mkpath "$aptdir/state/lists/partial";
	mkpath "$aptdir/cache/archives/partial";

	my $aptget   = "apt-get --assume-yes -o " . join(" -o ", @aptopts);

	print STDERR "aptget: $aptget\n" if $debug;
	system("$aptget update 1>&2");
    } elsif ("apt-cache" eq "$op") {
	my $aptcache = "apt-cache -o=" . join(" -o=", @aptopts);
	print STDERR "aptcache: $aptcache\n" if $debug;
	return $aptcache;
    }
}

sub load_available_packages
{
    apt("update");
    my $aptcache = apt("apt-cache");
    open(APT, "$aptcache dump |") || die "Unable to start apt-cache";
    my $pkg;
    while (<APT>) {
	chomp;
	if (/^Package: (.+)$/) {
	    $pkg = $1;
	    print STDERR "Found pkg '$pkg'\n" if $debug;
	}
	if (/^\s+Version:\s+(.+)/) {
	    print STDERR " pkg $pkg = ver $1\n" if $debug;
#	    print "C: $pkg $available{$pkg} lt $1\n" if ( exists $available{$pkg});
	    $available{$pkg} = $1 if ( ! exists $available{$pkg} ||
				       $available{$pkg} lt $1 );
	}
    }
}

sub load_tasks {
    my $line;
    while (<>) {
	chomp;
	$line = $_;

	# Append multi-line
	while ($line =~ /\\$/) {
	    $line =~ s/\s*\\//;
	    $_ = <>;
	    chomp;
	    $line .= $_;
	}
	# Remove trailing space
	$line =~ s/\s+$//;

	$_ = $line;
	if (m/^Package:\s+(.+)$/ && $1 !~ /^\s*$/) {
	    my $packages;
	    for $packages (split(/\s*,\s*/, $1)) {
		print "E: double comma?: $line\n" if ($packages =~ /^\s*$/ && $debug);
		my $package;
		for $package (split(/\s*\|\s*/, $packages)) {
	            print STDERR "Loading pkg '$package'\n" if $debug;
		    if ($package =~ /^-(.+)$/) {
			$excluded{$1} = 1;
		    } elsif ( !exists $available{$package} ) {
			push(@missing, $package);
		    } elsif ( ! $included{$package} ) {
			push(@wanted, $package);
			$included{$package} = 1;
		    } else {
			# nothing
		    }
		}
	    }
	}
	if (/^Missing:\s+(.+)$/) {
	    my @pkgs = split(/\s*,\s*/, $1);
	    my $packages;
	    for $packages (@pkgs) {
	        my $package;
	    	for $package (split(/\s*\|\s*/, $packages)) {
		    # Remove explanations, ie the paranteses at the end.
		    $package =~ s/\s*\([^\)]*\)\s*$//;
		    push(@missing, $package);
		}
	    }
	}
    }
}

sub print_excluded_packages {
    print join("\n", sort keys %excluded),"\n";
}

sub print_available_packages {
    print join("\n", @wanted),"\n";
}

sub print_all_packages {
    print STDERR "Printing all packages\n" if $debug;
    print join("\n", @wanted, @missing),"\n";
}

sub print_missing_packages {
    if (@missing) {
	print STDERR "Missing packages:\n";
	my $package;
	for $package (@missing) {
	    if (exists $available{$package}) {
	        print STDERR "  $package (v$available{$package} available)\n";
	    } else {
	        print STDERR "  $package\n";
	    }
	}
    }
}
